/* Laesst eine LED an PB0 und PB7 endlos blinken
 * Programm nicht optimal!
 */

#include <avr/io.h>
#include <util/delay.h>		// _delay_ms() 
#include "charset.h"
#include <avr/interrupt.h>

void delay_ms(uint16_t);
void set_row (uint8_t);
void write_row (uint8_t);
int main (void);

#define DISPCOLS 15			// Anzahl der Spalten
#define MEMCOLS 200			// Displayspeicher fuer 200 Spalten reservieren (>= 6 x Anzahl der Zeichen in Zeichenkette + 2 x DISPCOLS)
#define CHARWIDTH 5			// Spalten je Zeichen
#define CHARHEIGHT 7		// Zeilen je Zeichen
#define DELAY 50			// Textgeschwindigkeit (ca. 10..200: schnell..langsam)

volatile uint8_t memory[DISPCOLS];		// Speicher fr Display, spaltenweise orientiert, Index 0=links
volatile uint8_t pos=0;					// ab welcher Speicherposition soll dargestellt werden
volatile uint8_t zeile=0;				// Zeilenzhler welche Zeile aktuell ist
volatile uint8_t durchlauf=0;			// Anzahl der Durchlaeufe
volatile uint8_t chars=0;				// Anzahl der Zeichen in Zeichenkette
uint8_t text[]="Herzlich Willkommen!";	// Zeichenkette Ausgabe

/**
	@brief	Wird vom 16 Bit Timer ausgelst, wenn dieser den Vergleichwert erreicht
	@param	none
	@return	none
*/
SIGNAL (SIG_OUTPUT_COMPARE1A)
{
	write_row (zeile);			// schreibe aktuelle Zeile
	zeile++;					// nchste mal naechste Zeile
	if (zeile == CHARHEIGHT)	// wenn alle Zeilen ausgegeben, wieder von vorne beginnen
		zeile = 0;
	
	durchlauf++;				// Durchlaufzaehler: wie oft wurde das Display unverndert refresht
	if (durchlauf == DELAY)		// Anzahl der Durchlufe erreicht
	{
		durchlauf = 0;
		pos++;					// Displayinhalt eins verschieben
	}
	
	// Wenn der gesamte Speicherinhalt einmal durchlaufen wurde 
	if (pos >= (chars*(CHARWIDTH+1)+(DISPCOLS)))
		pos = 0;				// von vorne beginnen
		
}

// wartet ms Millisekunden
void delay_ms(uint16_t ms)
{
	for(uint16_t t=0; t<=ms; t++)
		_delay_ms(1); 
}

/**
	@brief	Schiebt die Daten fr die Zeile in die Schieberegister.
			Daten werden aus dem globalen Array memory genommen
	@param	Zeile (0..6)
	@return	none
*/
void set_row (uint8_t row)
{
	int8_t i;
	
	// Alle Spalten des Displays durchlaufen. Mit rechter Spalte beginnen
	for (i = DISPCOLS-1; i>=0; i--)		
	{
		// Bit in der Spalte + Verschiebungswert pos an der bergebenden Zeile gesetzt?
		if ((memory[i+pos] >> (6-row)) & 1)	// 
			PORTC |= (1 << PC1);	// Data Bit setzen
		else
			PORTC &= ~(1 << PC1);	// Data Bit lschen
		
		PORTC |= (1 << PC0);	// Takt HIGH -> Flanke
		PORTC &= ~(1 << PC0);	// Takt LOW
	}
}

void write_row (uint8_t row)
{
	PORTB = 0;			// alle Zeilen aus
	set_row (row);		// Daten fr aktuelle Zeile in Schieberegister schieben
	PORTB = (1 << row);	// aktuelle Zeile an
}

int main()
{
	uint8_t	i, row;

	DDRC  |= (1 << DDC0) | (1 << DDC1);		// PC0, PC1 als Ausgang
	PORTC &= ~((1 << PC0) | (1 << PC1));	// PC0, PC1 aus
	DDRB  = 0xFF; 							// PortB als Ausgang
	PORTB = 0;								// PortB aus

	// Leerspalten am Anfang, damit Text von rechts kommt
	for (row=0; row<DISPCOLS; row++)
		memory[row]=1;		// Eigentlich 0 aber fr besseres Verstndnis 1, damit eine LED leuchtet
		
	// Zeichenkette in Spaltenbeschreibung umsetzen
	while (text[chars] != 0)								// solange Zeichen in Zeichenkette 
	{
		memory[row]=charset[text[chars]-32][0]; row++;		// aktuelle Spalte = 1. Spalte des Fonts
		memory[row]=charset[text[chars]-32][1]; row++;		// aktuelle Spalte = 2. Spalte des Fonts
		memory[row]=charset[text[chars]-32][2]; row++;		// usw.
		memory[row]=charset[text[chars]-32][3]; row++;
		memory[row]=charset[text[chars]-32][4]; row++;
		memory[row]=0; row++;								// Leerspalte: Abstand zwischen 2 Buchstaben
		chars++;											// Anzahl der Zeichen zhlen
	}
	
	// Leerspalten am Ende, damit Textende aus Display geschoben wird
	for (i=0; i<DISPCOLS; i++)
		memory[i+row]=2;	// Eigentlich 0 aber fr besseres Verstndnis 1, damit eine LED leuchtet

	OCR1A = (uint16_t)(F_CPU/1024.0*0.003);			// Vergleichwert fr 0,003s laden: Takt= 8MHz/1024 = 7812,5 Zhlerschritte pro s
	TIMSK |= (1<<OCIE1A); 							// Interrupt wenn Timer Vergleichswert erreicht
	TCCR1B = _BV(CS12) | _BV(CS10) | _BV(WGM12);	// Timer mit Div 1024 starten und Clear Counter on Compare Match
	sei();											// IRQ Behandlung ein
	
	while (1)
	{
		asm volatile ("nop");		// nichts machen
	}

	return 0;
}
