/* Beispiel fuer Bootloader
 * Laesst eine LED an PD5 blinken. Taster1 springt in Application Flash
 * Der Bytecode soll im Bereich fuer den Bootloader gespeichert werden.
 * Dazu muss dem Linker die Option -tText= im makefile uebergeben werden.
 * 
 */

#include <avr/io.h>
#include <util/delay.h>		
#include <avr/interrupt.h>

// wartet ms Millisekunden
void delay_ms(uint16_t ms)
{
	for(uint16_t t=0; t<=ms; t++)
		_delay_ms(1); 
}

// ISR wird bei Tastendruck ausgefuehrt
ISR (INT0_vect)
{
	// IRQ Vektoren auf Bootloader verbiegen:
	cli();					// IRQs sperren
	GICR = (1 << IVCE);	// Interrupt Vector Change Enable
	GICR = 0;				// Interrupt Vector Select: Flash

	asm("jmp 0x0000");		// Sprung Anfang Flash
}

int main()
{
	// IRQ Vektoren auf Bootloader verbiegen:
	cli();					// IRQs sperren
	TCCR0=0;				// Timer Prescaler = STOP (Wichtig!)
	GICR = (1 << IVCE);	// Interrupt Vector Change Enable (jetzt nur 4 Ticks Zeit)
	GICR = (1 << IVSEL);	// Interrupt Vector Select: Bootloader

	PORTD |= (1 << PD5);	// LED an PD5 ein
	DDRD |= (1 << DDD5);	// D5 als Ausgang
	DDRD &= ~(1 << DDD2);	// PD2 als Eingang

	// IRQ bei Tastendruck:
	MCUCR = (1 << ISC00)|(1 << ISC01);	// steigende Flanke INT0 => IRQ
	GICR = (1 << INT0);				// IRQ an INT0 aktivieren
	sei();								// IRQs global zulassen
	
	while (1)
	{
		delay_ms (250);
		PORTD ^= (1 << PD5);	// toggle LED an PD5
	}	
	return 0;
}
