/* D/A Wandler an Port B, LCD an Port C
 * Digitaler Signalprozessor: Erzeugt verschiedene Signalformen
 * Steuerung ber serielle Schnittstelle
 *
 * Programm nicht optimal!
 */

#include <stdlib.h>			// atol
#include <avr/io.h>
#include <math.h>			// trigonometrische Funktionen
#include <util/delay.h>		// _delay_ms() 
#include "lcd_tools.h"
#include "uart.h"

// Lookup table Sinus
uint8_t value[] = 
{
0x80, 0x83, 0x86, 0x89, 0x8C, 0x8F, 0x92, 0x95, 0x99, 0x9C, 0x9F, 0xA2, 0xA5, 0xA8, 0xAB, 0xAE, 
0xB1, 0xB4, 0xB6, 0xB9, 0xBC, 0xBF, 0xC2, 0xC4, 0xC7, 0xC9, 0xCC, 0xCF, 0xD1, 0xD3, 0xD6, 0xD8, 
0xDA, 0xDC, 0xDF, 0xE1, 0xE3, 0xE5, 0xE7, 0xE8, 0xEA, 0xEC, 0xEE, 0xEF, 0xF1, 0xF2, 0xF3, 0xF5, 
0xF6, 0xF7, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFD, 0xFE, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFE, 0xFD, 0xFD, 0xFC, 0xFB, 0xFB, 0xFA, 0xF9, 0xF8, 0xF7, 
0xF5, 0xF4, 0xF3, 0xF1, 0xF0, 0xEE, 0xED, 0xEB, 0xE9, 0xE8, 0xE6, 0xE4, 0xE2, 0xE0, 0xDE, 0xDB, 
0xD9, 0xD7, 0xD5, 0xD2, 0xD0, 0xCD, 0xCB, 0xC8, 0xC6, 0xC3, 0xC0, 0xBD, 0xBB, 0xB8, 0xB5, 0xB2, 
0xAF, 0xAC, 0xA9, 0xA6, 0xA3, 0xA0, 0x9D, 0x9A, 0x97, 0x94, 0x91, 0x8E, 0x8B, 0x87, 0x84, 0x81, 
0x7E, 0x7B, 0x78, 0x74, 0x71, 0x6E, 0x6B, 0x68, 0x65, 0x62, 0x5F, 0x5C, 0x59, 0x56, 0x53, 0x50, 
0x4D, 0x4A, 0x47, 0x44, 0x42, 0x3F, 0x3C, 0x39, 0x37, 0x34, 0x32, 0x2F, 0x2D, 0x2A, 0x28, 0x26, 
0x24, 0x21, 0x1F, 0x1D, 0x1B, 0x19, 0x17, 0x16, 0x14, 0x12, 0x11, 0x0F, 0x0E, 0x0C, 0x0B, 0x0A, 
0x08, 0x07, 0x06, 0x05, 0x04, 0x04, 0x03, 0x02, 0x02, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 
0x0A, 0x0C, 0x0D, 0x0E, 0x10, 0x11, 0x13, 0x15, 0x17, 0x18, 0x1A, 0x1C, 0x1E, 0x20, 0x23, 0x25, 
0x27, 0x29, 0x2C, 0x2E, 0x30, 0x33, 0x36, 0x38, 0x3B, 0x3D, 0x40, 0x43, 0x46, 0x49, 0x4B, 0x4E, 
0x51, 0x54, 0x57, 0x5A, 0x5D, 0x60, 0x63, 0x66, 0x6A, 0x6D, 0x70, 0x73, 0x76, 0x79, 0x7C, 0x7F};

/**
	@brief	wartet Nanosekunden
	@param	Zeit in Nanosekunden
	@return	none
*/
void delay_us(uint32_t ms)
{
  for(uint32_t t=0; t<=ms; t++)
	_delay_us(1); 
}

int main()
{
	long i;
	unsigned char zeichen=0, eingabe[20];
	unsigned long pause=0;
	
	DDRB = 0xFF;		// PORTB (D/A) als Ausgang
	uart_ini();
	lcd_ini();
	
	lcd_writetext (" Funktionsgenerator");			// Auf LCD
	uart_puts ("\033[;H\033[2J");					// an RS232: Terminal Bildschirm lschen (VT-100)
	uart_puts ("*** Funktionsgenerator ***\r\n");	
	uart_puts ("[z]=Saegezahn ansteigend, [Z]=Saegezahn absteigend, [d]=Dreieck,\r\n");
	uart_puts ("[r]=Rechteck, [s]=Sinus lookup, [S]=Sinus math.h, [x]=Sinus lookup schnell\r\n\r\n");	
	uart_puts ("> ");
	
	while (1)
	{
		if (rbuffcnt)		// Zeichen empfangen
		{
			zeichen = ser_getc();
			switch (zeichen)
			{
				case 'z' : 	lcd_gotoline (3);
							lcd_writetext ("Saegezahn ansteigend");
							break;
				case 'Z' : 	lcd_gotoline (3);
							lcd_writetext ("Saegezahn absteigend");
							break;
				case 'd' : 	lcd_gotoline (3);
							lcd_writetext ("Dreieck             ");
							break;
				case 'r' : 	lcd_gotoline (3);
							lcd_writetext ("Rechteck            ");
							break;
				case 's' : 	lcd_gotoline (3);
							lcd_writetext ("Sinus lookup table  ");
							break;
				case 'x' : 	lcd_gotoline (3);
							lcd_writetext ("Sinus lookup schnell");
							break;
				case 'S' : 	lcd_gotoline (3);
							lcd_writetext ("Sinus math.h        ");
							break;
				default  : 	PORTB=0;						// Signal aus; Empfang von Eingaben, die versucht werden, als Zahlenwert zu interpretieren
							lcd_gotoline (3);
							lcd_writetext ("                    ");		// Zeile loeschen
							i=0;
							while (zeichen != '\n')			// bis \r\n empfangen wurde
							{
								if (zeichen == '\r')
									zeichen = '\0';			// \r durch Stringendekennzeichen ersetzen => String fertig. Nur noch \n aus Empfangsbuffer abholen
								eingabe[i] = zeichen;		// String erweitern
								zeichen = ser_getc();		// nchstes Zeichen empfangen
								i++;
							}
							pause = atol (eingabe);			// Eingabestring in intger wandeln
							lcd_gotoline (4);
							lcd_writetext ("                    ");		// Zeile loeschen
							lcd_gotoline (4);
							lcd_writetext ("Delay: ");
							lcd_writetext (utoa (pause, eingabe, 10));	// Wert zurck in String konvertieren, 
																		// damit die tatschlich genutzte Zahl gezeigt wird
																		// und nicht der Enigabestring
							break;
			}
			uart_puts ("\n\r> ");
		}
	
		// Signal ausgeben
		switch (zeichen)
		{
			// Saegezahn ansteigend
			case 'z' : 	for (i = 0; i<=255; i++)
						{
							PORTB = i;
							delay_us (pause);
						}
						break;
			// Saegezahn absteigend
			case 'Z' : 	for (i = 255; i>=0; i--)
						{
							PORTB = i;
							delay_us (pause);
						}
						break;
			// Dreieck
			case 'd' : 	for (i = 0; i<=255; i++)
						{
							PORTB = i;
							delay_us (pause);
						}
						for (i = 255; i>=0; i--)
						{
							PORTB = i;
							delay_us (pause);
						}
						break;
			// Rechteck
			case 'r' : 	PORTB = 0xFF;
						_delay_us (40);		// etwas Pause, damit Rechteck sauber wird auch wenn Veroegrung=0
						delay_us (pause);
						PORTB = 0;
						_delay_us (40);
						delay_us (pause);
						break;
			// Sinus lookup table
			case 's' : 	for (i = 0; i<=255; i++)
						{
							PORTB = value[i];
							delay_us (pause);
						}
						break;
			// Sinus lookup table ohne jedwede Verzoegerung
			case 'x' : 	for (i = 0; i<=255; i++)
						{
							PORTB = value[i];
						}
						break;
			// Sinus trigonometrische Funktionen
			case 'S' : 	for (i = 0; i<=359; i++)
						{
							PORTB = (sin (i*M_PI/180)*127)+127;		// Bogenma in Gradma umrechnen, dann Sinus => langsam
							delay_us (pause);
						}
						break;
			default  :  PORTB=0;	// aus
						break;
		}
			
	}
	return 0;
}
