/* Empfngt GNSS Daten per Soft UART an D0 und sendet sie an den USART
 *
 * 19200Bd, 8N1
 *
 */

#include <string.h>
#include <stdlib.h>
#include <avr/io.h>
#include <util/delay.h>		// _delay_ms() 
#include <stdio.h>			// sprintf
#include "uart.h"
#include "softuart.h"

void getTeilstring (unsigned char *, unsigned char *, uint8_t, char, char);
void delay_ms(uint16_t);
void getGPS (char *);
int main(void);
uint8_t gpgsv=0;			// wurde GPGSV Datensatz empfangen?

/**
	@brief	Sucht im Quellstring nach dem n-ten vorkommen des Zeichen compare1 und kopiert den String ab 
			dem darauffolgenden Zeichen bis zum Zeichen compare2 (ohne dieses).
			Liefert leeren String, wenn zwischen beiden Suchzeichen kein Zeichen steht oder StartZeichen nicht gefunden
	@param	Zielstring
			Quellstring
			ab dem wievielten Start-Zeichen kopieren?
			welches Zeichen als Start suchen?
			welches Zeichen als Ende suchen
	@return	none
*/
void getTeilstring (unsigned char *dest, unsigned char *from, uint8_t pos, char compare1, char compare2)
{
	uint8_t	detect=0,			// gesuchtes n-tes vorkommen des Zeichen gefunden = 1
			cnt=0;
	
	while (*from != '\0')		// Zeichenkette durchlaufen bis Ende
	{
		if (*from == compare1)	// wenn gesuchtes Zeichen gefunden wurde
			cnt++;				// das wievielte mal wurde das gesuchte Zeichen gefunden?
		if (cnt == pos)			// wenn das gesuchte Zeichen zum n-ten mal gefunden wurde: kopiere String bis zum naechsten auftreten
		{
			compare1=compare2;	// ab jetzt bis zum Ende-Zeichen suchen
			if (detect == 0)	// Bisher kein Flag gesetzt
				detect = 1;		// Flag dafr, dass das Zeichen zum n-ten mal gefunden wurde setzen
			else				// nur wenn das Flag gesetzt wurde, dann die Zeichen kopieren. 
			{					// So wird das gesuchte Zeichen selbt nicht mitkopiert
				*dest = *from;	// Zeichen kopieren
				dest++;			// naechstes Zeichen im Zielstring waehlen
			}
		}
		from++;					// naechstes Zeichen im Quellstring waehlen
	}
	*dest = '\0';				// Zielstring mit Endmarkierung versehen
}

// wartet ms Millisekunden
void delay_ms(uint16_t ms)
{
	for(uint16_t t=0; t<=ms; t++)
		_delay_ms(1); 
}

/**
	@brief	Liest Daten vom GNSS Modul aus und wertet diese aus. 
			Schreibt die Daten mit VT100 Escapesequenzen an die serielle Schnittstelle
	@param	Zeichenarray welches den unveraenderten Datensatz enthaelt, der empfangen wurde
	@return	none
*/
void getGPS (char *buffer)
{
	uint8_t	u=0, 
			byte;
	char 	
			tmp[170],
			gpscoord[170];

	do		// Zeichen von Pin per Soft UART einlesen bis \n empfangen wird
	{
		byte = softuart_getc();		// Zeichen einlesen
		buffer[u]=byte;				// In Puffer schreiben
		u++;						// Zeiger auf Position im Puffer
	}
	while (byte != '\n');			// bis \n empfangen

	buffer[u-2]='\0';				// vorletztes Zeichen (CR) durch Stringendekennung ersetzen
	u=0;
	byte=0;

	if (strncmp  (buffer, "$GPGSV", 6) == 0)					// GPGSV Datensatz empfangen => GPGGA Datensatz enthaelt echte GPS Daten
		gpgsv = 1;
	
	if ((strncmp  (buffer, "$GPGGA", 6) == 0) && gpgsv) 		// GPGGA Datensatz empfangen und Daten auch von GPS
	{
		gpgsv = 0;					// fuer naechsten Durchlauf erst wieder auf GPGSV warten

		uart_puts ("\e[1;10H");		// gehe zu Pos. y,x
		uart_puts ("\e[K");			// Loesche bis Zeilenende
		uart_puts ("\e[31m");		// Rot ein

		getTeilstring (tmp, buffer, 1, ',', ',');			// = Uhrzeit UTC
		sprintf (gpscoord, "%c%c:%c%c:%c%c", tmp[0], tmp[1], tmp[2], tmp[3], tmp[4], tmp[5]);		// "hh:mm:ss "
		uart_puts ((char*)gpscoord);

		getTeilstring (tmp, buffer, 6, ',', ',');			// = Qualitaet
		if (tmp[0] != '0')									// Qualitaet 0=ungueltig, sonst OK
		{
			
			getTeilstring (tmp, buffer, 3, ',', ',');		// = Hemisphre
			sprintf ((char*)gpscoord, " * %s ", tmp);
			getTeilstring (tmp, buffer, 2, ',', ',');		// = Latitude (z. B. 5256.0361)
			sprintf ((char*)gpscoord, "%s%c%c%c%c%c%c%c%c%c", gpscoord, tmp[0], tmp[1], tmp[2], tmp[3], tmp[4], tmp[5], tmp[6], tmp[7], tmp[8]);

			getTeilstring (tmp, buffer, 5, ',', ',');		// = Hemisphre
			sprintf ((char*)gpscoord, "%s - %s ", gpscoord, tmp);
			getTeilstring (tmp, buffer, 4, ',', ',');		// = Longitude (z. B. 01248.0766)
			sprintf ((char*)gpscoord, "%s%c%c%c%c%c%c%c%c%c%c", gpscoord, tmp[0], tmp[1], tmp[2], tmp[3], tmp[4], tmp[5], tmp[6], tmp[7], tmp[8], tmp[9]);

			uart_puts ((char*)gpscoord);					// Ausgabe bearbeitete Daten
			
			uart_puts ("  ~  ");
			uart_puts ((char*)buffer);						// Ausgabe urspruenglicher Datensatz

		}
	}		
	else if ((strncmp  ((char*)buffer, "$GNRMC", 6) == 0) || (strncmp  ((char*)buffer, "$GPRMC", 6) == 0))		// G?RMC Datensatz empfangen
	{
		
		if (strncmp  ((char*)buffer, "$GNRMC", 6) == 0)		// Positioniere Cursor je nach Datensatz
		{
			uart_puts ("\e[2;10H");			// gehe zu Pos. y,x
			uart_puts ("\e[K");				// Loesche bis Zeilenende
			uart_puts ("\e[36m");			// Cyan ein
		}
		else
		{
			uart_puts ("\e[1;10H");			// gehe zu Pos. y,x
			uart_puts ("\e[K");				// Loesche bis Zeilenende
			uart_puts ("\e[31m");			// Rot ein

		}

		getTeilstring (tmp, buffer, 1, ',', ',');			// = Uhrzeit UTC
		sprintf ((char*)gpscoord, "%c%c:%c%c:%c%c", tmp[0], tmp[1], tmp[2], tmp[3], tmp[4], tmp[5]);		// "hh:mm:ss "
		uart_puts ((char*)gpscoord);
			
		getTeilstring (tmp, buffer, 2, ',', ',');			// = Status A/V
		if (tmp[0] == 'A')									// Empfaengerwarnung A=OK
		{
			getTeilstring (tmp, buffer, 4, ',', ',');		// = Hemisphre
			sprintf ((char*)gpscoord, " * %s ", tmp);
			getTeilstring (tmp, buffer, 3, ',', ',');		// = Latitude (z. B. 5256.0361)
			sprintf ((char*)gpscoord, "%s%c%c%c%c%c%c%c%c%c", gpscoord, tmp[0], tmp[1], tmp[2], tmp[3], tmp[4], tmp[5], tmp[6], tmp[7], tmp[8]);

			getTeilstring (tmp, buffer, 6, ',', ',');		// = Hemisphre
			sprintf ((char*)gpscoord, "%s - %s ", gpscoord, tmp);
			getTeilstring (tmp, buffer, 5, ',', ',');		// = Longitude (z. B. 01248.0766)
			sprintf ((char*)gpscoord, "%s%c%c%c%c%c%c%c%c%c%c", gpscoord, tmp[0], tmp[1], tmp[2], tmp[3], tmp[4], tmp[5], tmp[6], tmp[7], tmp[8], tmp[9]);
			
			uart_puts ((char*)gpscoord);					// Ausgabe bearbeitete Daten
			
			uart_puts ("  ~  ");
			uart_puts ((char*)buffer);						// Ausgabe urspruenglicher Datensatz
		}
	}
}

int main(void)
{
	char 	buffer[170];

	DDRA &= ~(1 << DDA0); 				// PA0 Eingang RxD GPS

	uart_ini();
	softuart_init();
	
	uart_puts ("\ec");					// Reset Terminal (CLS)
	uart_puts ("\e[7m");				// Invers ein
	uart_puts ("GPS    :\r\n");
	uart_puts ("GLONASS:\r\n");
	uart_puts ("Receive:");
	uart_puts ("\e[27m");				// Invers aus

	while (1)
	{
		getGPS(buffer);
		
		uart_puts ("\e[3;10H");			// gehe zu Pos. y,x
		uart_puts ("\e[K");				// Loesche bis Zeilenende
		uart_puts ("\e[44m");			// Hintergrund blau ein
		uart_puts ("\e[33m");			// Gelb ein
		uart_puts ((char*)buffer);		// Ausgabe urspruenglicher Datensatz
		uart_puts ("\e[0m");			// Alle Attribute aus.
	}
	
	return 0;
} 

