/* Sendet SMS im PDU Format an ein Handy
 *
 * 9600Bd, 8N1
 *
 */

#include <stdint.h>
#include <stdio.h>

#include "uart.h"

uint8_t SMS_send (char *, char *, uint8_t);
int main(void);

/**
	@brief	sendet eine SMS ohne Angabe des SMSC im PDU Format an die angegeben 
			Telefonnumer im internationalen Format (+49 123 123456).
			Oder speichert sie im Ausgangsordner des Handy
	@param	Telefonnummer (ohne das Plus und ohne Leerzeichen)
			SMS Nachricht (nur ASCII 7 Bit Zeichen nutzen)
			Senden = 0, Speichern = 1
	@return	Erfolg = 0, Fehler = 1
*/
uint8_t SMS_send (char *zielnr, char *nachricht, uint8_t typ)
{
	uint8_t	anz_septents = 0,	// Anzahl der Septets fr den Nachrichtenstring
			anz_bytes = 1,		// Anzahl bytes des Nachrichtentextes inkl. Lngeninfobyte
			i = 0,
			nachricht2[161],
			zielnr2[20],
			shift = 0,	
			string[200],
			erg=1;
	char 	zeichen1 = 0, 
			zeichen2 = 0;
	
	// Nachrichten String erstellen
	while (nachricht[anz_septents] != 0)		// solange Zeichen in der Nachricht
	{
		nachricht2[anz_bytes] = (nachricht[anz_septents]&0x7F) >> shift;
		if (shift != 0)
			nachricht2[anz_bytes-1] |= (nachricht[anz_septents]&((1<<shift)-1)) << (8-shift);
		shift++;
		anz_septents++;
		if (shift == 8)
			shift=0;
		else
			anz_bytes++;
	}
	nachricht2[0] = anz_septents;		// Lngeninfo (Anzahl septets) vor den Nachrichtentext schreiben
	
	// Zielnummer codieren
	while (zielnr[i] !=0)		// solange Zeichen in Nummernstring
	{
		if (zielnr[i+1] == 0)		// wenn ungerade Lnge der Zeilnummer, dann wird hier auf Stringendekennung gezeigt
			zielnr2[i] = 'F';		// dann als vorletztes Zeichen in der konvertierten Zielnummer ein F schreiben
		else
			zielnr2[i] = zielnr[i+1];
		zielnr2[i+1] = zielnr[i];
		i += 2;
	}
	zielnr2[i] = 0;			// Stringendekennzeichnung anhngen

    if (typ == 0)			// SMS senden
		sprintf(string, "at+cmgs=");		// Sende AT-Befehl: Versenden von Kurznachrichten (PDU-Mode) 
	else if (typ == 1)		// SMS speichern
		sprintf(string, "at+cmgw=");		// Sende AT-Befehl: Speichern von Kurznachrichten (PDU-Mode) 
    uart_puts(string);					// An USART schicken

    sprintf(string, "%i\r\n", (anz_bytes+(i/2)+7));		//  Lnge der PDU
    uart_puts(string);					// An USART schicken

	while(ser_getc()!=' ');				// warte auf Leerzeichen ("> ");
	
	sprintf(string, "00");				// keine SMSC-Adresse angeben (Byte wird bei der Lngenberechnung nicht bercksichtig)
    uart_puts(string);					// An USART schicken
	sprintf(string, "11");				// Header Message Flags => SMS Submit, Reject Duplicates aus, Validity Period: not present, Status Report Request aus, no ser Data Header, no Reply Path
    uart_puts(string);					
	sprintf(string, "00");				// Message Reference Number 00 => Telefon vergibt selbstndig eine eigene Referenznummer
    uart_puts(string);					
	if(zielnr2[i-2] == 'F')				// wenn ungerade Lnge der Zielnummer, vorletztes Zeichen ist ein F => Anzahl eins geringer, da nur die Zahl der tatschlichen Zahlen der Nummer angegeben wird
		i--;
	sprintf(string, "%02X91", i);		// zweistellig, hexadezimal Anzahl Bytes der Nummer (i) + "91" fr Internationale Nummernangabe ("+49...")
    uart_puts(string);					
	sprintf(string, zielnr2);			// Zielnummer
    uart_puts(string);					
	sprintf(string, "00");				// Protocol Identifier
    uart_puts(string);					
	sprintf(string, "00");				// Data Coding Scheme, 00 = 7 Bit
    uart_puts(string);					
	sprintf(string, "AA");				// Expire time: 4 days
    uart_puts(string);					
	for (anz_septents = 0;anz_septents<anz_bytes;anz_septents++)
	{
		sprintf(string, "%02X", nachricht2[anz_septents]);
		uart_puts(string);					
	}
    sprintf(string, "%c",0x1A);			// CTRL-Z beendet PDU
    uart_puts(string);					
	
	// lese Antwort von Handy ein: 1 Zeile bei Fehler, 3 Zeilen bei OK
	while (ser_getc() != 10);		// lese Zeichen bis Linefeed ein (\r\n) als Antwort auf CTRL-Z
	do
	{
		zeichen2 = ser_getc();		// Antwort abholen
		if ((zeichen2 == 'R') && (zeichen1 == 'E'))		// Dieses Zeichen = "R" und vorheriges = "E" => "ERROR"
		{
			erg = 1;		// Fehler
		}
		if ((zeichen2 == 'G') && (zeichen1 == 'M'))		// Dieses Zeichen = "G" und vorheriges = "M" => "+CMGW" oder "+CMGS"
		{
			erg = 0;		// OK
			while (ser_getc() != 10);		// lese Zeichen bis Linefeed ein (= 1. Zeile zu Ende)
			while (ser_getc() != 10);		// lese Zeichen bis Linefeed ein (= 2. Zeile)
		}
		zeichen1 = zeichen2;
	}
	while (zeichen1 != 10);		// bis Linefeed (bei Fehler nur 1 Zeile, bei OK 3 Zeilen wobei schon 2 eingelesen wurden)
	return erg;
}


int main(void)
{
	uart_ini();
 
	SMS_send ("49123123456", "Testtext", 1);

	return 0;
} 

