/**
	Beispiel zur Ansteuerung eines LCDs mit ST7735 Controller
 */


#define __PROG_TYPES_COMPAT__
#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include "tft_st7735.h"

// Zeichnet einen Reglerbalken
void draw_regler (uint8_t wert)
{
	char buf[5];

	setCursor(40,52);
	setTextSize(3);
	setTextColor(Color565(190,40,40),Color565(160,205,255));
	if (wert > 0)
		fillRect(16,31,wert,17,Color565(65,200,5));				// Reglerbalken
	if (wert < 97)
		fillRect(16+wert,31,97-wert,17,Color565(160,205,255));		// Rest Inhalt lschen
	setCursor(50,52);
	sprintf(buf, "%d ", wert);		// Leerzeichen wichtig, damit einstellige Zahlen vorherige zweistellige ueberschreiben
	print(buf);
}

int main(void)
{
	uint8_t i;
	
	init();				// init the 1.8 lcd display
	DDRD &= ~(1 << DDD2) & ~(1 << DDD3) & ~(1 << DDD4);	// Taster als Eingaenge. JP setzen.

	while(1)
	{
		// Farbflaechen
		setTextSize(1);
		fillRect(0,0,64,40,Color565(255,0,0));
		setTextColor(Color565(255,255,255),Color565(255,0,0));
		setCursor(15,15);
		print("red");
		
		fillRect(64,0,64,40,Color565(0,255,0));
		setTextColor(Color565(255,255,255),Color565(0,255,0));
		setCursor(80,15);
		print("green");
		
		fillRect(0,40,64,40,Color565(0,0,255));
		setTextColor(Color565(255,255,255),Color565(0,0,255));
		setCursor(15,55);
		print("blue");
		
		fillRect(64,40,64,40,Color565(255,255,0));
		setTextColor(Color565(255,255,255),Color565(255,255,0));
		setCursor(80,55);
		print("yellow");
		
		fillRect(0,80,64,40,Color565(255,0,255));
		setTextColor(Color565(255,255,255),Color565(255,0,255));
		setCursor(15,95);
		print("magenta");
		
		fillRect(64,80,64,40,Color565(0,255,255));
		setTextColor(Color565(255,255,255),Color565(0,255,255));
		setCursor(80,95);
		print("cyan");
		
		fillRect(0,120,64,40,Color565(0,0,0));
		setTextColor(Color565(255,255,255),Color565(0,0,0));
		setCursor(15,135);
		print("black");
		
		fillRect(64,120,64,40,Color565(255,255,255));
		setTextColor(Color565(0,0,0),Color565(255,255,255));
		setCursor(80,135);
		print("white");
		
		drawRect(6,6,118,150,Color565(50,50,50));			// "Schatten"
		drawRect(5,5,118,150,Color565(255,255,255));
		
		_delay_ms(5000);

		// Linienmuster
		fillScreen(Color565(0,0,0));

		for (uint8_t y=0; y<160; y+=8)
		{
			drawLine(0, 0, 127, y, Color565(255,0,0));	
			drawLine(0, 159, 127, y, Color565(0,255,0));
			drawLine(127, 0, 0, y, Color565(0,0,255));
			drawLine(127, 159, 0, y, Color565(255,255,255));
		}
		_delay_ms(5000);
		
		// Textausgabe		
		fillScreen(Color565(0,0,0));
		setCursor(0,0);
		setTextWrap(1);
		setTextSize(2);
		setTextColor(Color565(255,255,0),Color565(0,0,255));
		print("All available chars:\n");

		setTextSize(1);
		setTextColor(Color565(255,255,255),Color565(0,0,0));
		for (i=0; i<255; i++) 
		{
			write(i);
		}
		_delay_ms(5000);

		// Volumeregler		
		fillScreen(Color565(160,205,255));
		setCursor(5,5);
		setTextWrap(1);
		setTextSize(2);
		setTextColor(Color565(255,90,5),Color565(160,205,255));
		print("Volume:");

		drawRect(15,30,100,20,Color565(50,50,50));			// "Schatten"
		drawRect(14,29,100,20,Color565(255,255,255));

		// Einmal von leer bis voll durchlaufen
		for (i=0; i<=97; i++)
		{
			draw_regler (i);
			_delay_ms (5);
		}

		_delay_ms(500);
		
		setCursor(0,80);
		setTextSize(1);
		setTextColor(Color565(0,0,0),Color565(160,205,255));
		print(" Press\n Taster1 for -\n Taster2 for +\n Taster3 for end");

		i=45;
		draw_regler (i);
		do
		{
			if (PIND & (1 << PIND2))	// Taster1: -
			{
				if (i>0)
				{
					i--;
					draw_regler (i);
				}
			}
			else if (PIND & (1 << PIND3))	// Taster2: +
			{
				if (i<97)
				{
					i++;
					draw_regler (i);
				}
			}
			_delay_ms(50);					// Autorepeat etwas verlangsamen
		} while(!(PIND & (1 << PIND4)));	// bis Taster3
	}

	return 0;
}