#include <avr/pgmspace.h>

/******************** PORT AND PINS ************************************
 * change these according to your needs
 *
 *	// ATMEGA 16 with hardware SPI
 * SCL      PB7 (SCK)
 * SDA      PB5 (MOSI)
 * RS       PD7 ("A0", "D/C")
 * RST      PD6 (Reset)
 * CS       PB1 (Slave/Chip Select)
*/

#define RSREG DDRD
#define RSTREG DDRD
#define RSPORT PORTD
#define RSTPORT PORTD
#define RS PD7
#define RST PD6
#define SPIREG DDRB
#define SPIPORT PORTB
#define SCK PB7
#define MOSI PB5
#define CS PB1

// Display size:
#define _width    128
#define _height   160

int16_t  cursor_x, cursor_y;
uint16_t textcolor, textbgcolor;
uint8_t  wrap; 							// If set, 'wrap' text at right edge of display
uint8_t textsize;						// definiert die Textgroesse als Faktor, 1=normal und dann vielfache

/*
 * available functions:
 */
void init(void);																					// Beim Start einmal aufrufen
void setAddrWindow(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1);								// Hilfsfunktion
void fillScreen(uint16_t color);																	// Bildschirm fuellen mit Farbe (Clear Screen)
void drawPixel(int16_t x, int16_t y, uint16_t color);												// Ein Pixel ausgeben. X-, y-Position, Farbe
void drawFastVLine(int16_t x, int16_t y, int16_t h, uint16_t color);								// Hilfsfunktion
void drawFastHLine(int16_t x, int16_t y, int16_t w, uint16_t color);								// Hilfsfunktion
void fillRect(int16_t x, int16_t y, int16_t w, int16_t h,uint16_t color);							// gefuelltes Rechteck zeichnen. x-, y-Position, Breite, Hoehe, Farbe
void invertDisplay(unsigned char i);																// Display invertieren (0=Normal, 1=invertiert)
void drawChar(int16_t x, int16_t y, unsigned char c,uint16_t color, uint16_t bg, uint8_t size);	// Hilfsfunktion
void drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint16_t color);						// Linie zeichnen. Start x-, Start y-Position, Ziel X-, Ziel Y-Position, Farbe
void drawRect(int16_t x, int16_t y, int16_t w, int16_t h, uint16_t color);							// Rechteck zeichnen. x-, y-Position, Breite, Hoehe, Farbe

void print(const char str[]);						// Textstring an Cursorposition ausgeben
void write(uint8_t);								// Zeichen an Cursorposition ausgeben

void setCursor(int16_t x, int16_t y);				// Cursorposition festlegen. x-, y-Position
void setTextColor(uint16_t c, uint16_t bg);			// Textfarbe festlegen. Vorder-, Hintergrund
void setTextWrap(uint8_t w);						// Zeilenumbruch aktivieren (0=aus, 1=ein)
void setTextSize(uint8_t s);						// Zeichengroesse. Ganzzahliger Faktor. 1=nromal


// Pass 8-bit (each) R,G,B, get back 16-bit packed color
#define Color565(r,g,b) (uint16_t)((b & 0xF8) << 8) | ((g & 0xFC) << 3) | (r >> 3)

/*
 * do not touch the following defines if you do not know
 * what you are doing
 */

/********************** START SPI STUFF *********************************/
#define LSBFIRST 0
#define MSBFIRST 1

#define SPI_CLOCK_DIV4 0x00
#define SPI_CLOCK_DIV16 0x01
#define SPI_CLOCK_DIV64 0x02
#define SPI_CLOCK_DIV128 0x03
#define SPI_CLOCK_DIV2 0x04
#define SPI_CLOCK_DIV8 0x05
#define SPI_CLOCK_DIV32 0x06

#define SPI_MODE0 0x00
#define SPI_MODE1 0x04
#define SPI_MODE2 0x08
#define SPI_MODE3 0x0C

#define SPI_MODE_MASK 0x0C  // CPOL = bit 3, CPHA = bit 2 on SPCR
#define SPI_CLOCK_MASK 0x03  // SPR1 = bit 1, SPR0 = bit 0 on SPCR
#define SPI_2XCLOCK_MASK 0x01  // SPI2X = bit 0 on SPSR
/*********************** END SPI STUFF **********************************/

#define INITR_GREENTAB 0x0
#define INITR_REDTAB   0x1

#define ST7735_TFTWIDTH  128
#define ST7735_TFTHEIGHT 160

#define ST7735_NOP     0x00
#define ST7735_SWRESET 0x01
#define ST7735_RDDID   0x04
#define ST7735_RDDST   0x09

#define ST7735_SLPIN   0x10
#define ST7735_SLPOUT  0x11
#define ST7735_PTLON   0x12
#define ST7735_NORON   0x13

#define ST7735_INVOFF  0x20
#define ST7735_INVON   0x21
#define ST7735_DISPOFF 0x28
#define ST7735_DISPON  0x29
#define ST7735_CASET   0x2A
#define ST7735_RASET   0x2B
#define ST7735_RAMWR   0x2C
#define ST7735_RAMRD   0x2E

#define ST7735_PTLAR   0x30
#define ST7735_COLMOD  0x3A
#define ST7735_MADCTL  0x36

#define ST7735_FRMCTR1 0xB1
#define ST7735_FRMCTR2 0xB2
#define ST7735_FRMCTR3 0xB3
#define ST7735_INVCTR  0xB4
#define ST7735_DISSET5 0xB6

#define ST7735_PWCTR1  0xC0
#define ST7735_PWCTR2  0xC1
#define ST7735_PWCTR3  0xC2
#define ST7735_PWCTR4  0xC3
#define ST7735_PWCTR5  0xC4
#define ST7735_VMCTR1  0xC5

#define ST7735_RDID1   0xDA
#define ST7735_RDID2   0xDB
#define ST7735_RDID3   0xDC
#define ST7735_RDID4   0xDD

#define ST7735_PWCTR6  0xFC

#define ST7735_GMCTRP1 0xE0
#define ST7735_GMCTRN1 0xE1

