/* Demonstriert den Sleep Mode
 * 8-Bit Timer 0: CTC Modus laesst LED blinken
 * Low Level an INT0 weckt Controller wieder auf
 *
 * SLEEP_MODE_IDLE        
 * SLEEP_MODE_ADC         
 * SLEEP_MODE_PWR_DOWN    
 * SLEEP_MODE_PWR_SAVE    
 * SLEEP_MODE_STANDBY     
 * SLEEP_MODE_EXT_STANDBY 
 *
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>

#define TEILER 10				// Teilungsverhaeltnis
volatile uint8_t zaehler = TEILER;

// wartet ms Millisekunden
void delay_ms(uint16_t ms)
{
	for(uint16_t t=0; t<=ms; t++)
		_delay_ms(1); 
}

// Timer/Counter0 Compare Match
// LED blinken lassen
ISR (TIMER0_COMP_vect)
{
	zaehler--;					// Inkrementieren
	if (!zaehler)				// wenn zaehler keinen Wert hat (=0)
	{
		PORTD ^= (1 << PD5);	// PD5 (LED1) togglen
		zaehler = TEILER;		// wieder n Schritte
	}
}

// Externer Interrupt INT0 
ISR(INT0_vect) 
{
	GICR &= ~(1 << INT0);		// externen Interrupt sperren falls LOW Puls an INT0 lange
	
	// Die Interruptroutine kann leer sein, muss aber existieren!
	// Sonst springt der AVR nach dem Aufwachen zum Reset, weil kein sinnvoller
	// Interruptvektor eingetragen ist!
}


int main()
{
	DDRD |= (1 << DDD5);		// D5 als Ausgang
	DDRD &= ~(1 << DDD2);		// D2 (INT0) als Eingang
	PORTD |= (1 << DDD2);		// Pull-Up aktivieren
	
	// Timer0 fuer blinken von LED1
	TIMSK |= (1 << OCIE0);		// IRQ bei Compare Match
	OCR0 = 194;					// Vergleichswert einstellen => 40,07 Hz bei Q=16 Mhz
	TCCR0 |= (1 << WGM01);		// CTC-Modus
	TCCR0 |= (1 << CS00) | (1 << CS02);		// Prescaler: 1/1024

	MCUCR &= ~(1 << ISC00) | ~(1 << ISC01);	// Low Level an INT0 (D2) => IRQ
	
	sei();						// IRQs enable

	while (1)					// endlos
	{
		delay_ms (5000);			// 5 Sekunden warten (LED blinkt)
		
		GICR |= (1 << INT0);		// externen Interrupt an INT0 freigeben
		
		set_sleep_mode(SLEEP_MODE_PWR_DOWN);	// Sleep Mode waehlen
		sleep_mode();							// in den Schlafmodus wechseln
 
		// C wacht auf und setzt Programmausfuehrung hier fort
	}
	
	return 0;
}
