/* Serial Peripheral Interface
 *
 * Master
 *
 */

#include <avr/io.h>
#include "lcd_tools.h"

#define DDR_SPI		DDRB	// SPI Portrichtungsregister
#define DD_MOSI		DDB5	// SPI Pin MOSI
#define DD_MISO		DDB6	// SPI Pin MISO
#define DD_SCK		DDB7	// SPI Pin Clock
#define DD_SS		DDB4	// SPI Pin Slave Select

void SPI_MasterInit(void);
void SPI_MasterTransmit(char);
char SPI_MasterReceive(void);
int main(void);

/**
	@brief	Master Initialisierung: Portrichtung und Uebertragungsparameter setzen
	@param	none
	@return	none
*/
void SPI_MasterInit(void)
{
	DDR_SPI |= (1<<DD_MOSI) | (1<<DD_SCK) | (1<<DD_SS);		// SS, MOSI und SCK Ausgabe
	DDR_SPI &= ~(1<<DD_MISO);								// MISO Eingang
	
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1);	// Enable SPI, Master, clock rate fck/64
}

/**
	@brief	Byte senden
	@param	Byte
	@return	none
*/
void SPI_MasterTransmit(char data)
{
	SPDR = data;					// Uebertragung starten
	while(!(SPSR & (1<<SPIF)));		// warte bis Uebertragung abgeschlossen
}

/**
	@brief	Byte empfangen
	@param	none
	@return	Byte
*/
char SPI_MasterReceive(void)
{
	while(!(SPSR & (1<<SPIF)));		// warte bis Empfang komplett
	return SPDR;					// empfangenes Byte zurueckgeben
}

int main(void)
{
	char zahl;
	
	lcd_ini();
	lcd_writetext ("    *** SPI ***");
	
	DDRB |= (1 << DDB0);		// Slave Select Pin zum ATmega8 als Ausgang
	PORTB &= ~(1 << PB0);		// PB0 = 0 => Slave Select ATmega8
	
	SPI_MasterInit();			
	SPI_MasterTransmit(1);
	zahl = SPI_MasterReceive();

	PORTB |= (1 << PB0);		// PB0 = 1 => Slave un-Select ATmega8

	lcd_gotoline (2);
	lcd_writetext ("Emfangen: ");
	lcd_writezahl (zahl, 3, 0);
	
	return 0;
}



