/* Entfernungsmessung mit Ultraschallsensor HS-SR04
 * Nutzt den 16-Bit Timer 1 und Input Capture ICP1
 *
 * Im makefile fuer Floatingpoint sprintf aktivieren: PRINTF_LIB = $(PRINTF_LIB_FLOAT)
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>				// sprintf
#include "lcd_tools.h"

// Trigger
#define PORT PORTD
#define DDR DDRD
#define TRIGGER PD0

// Trigger steuern
#define OUTPUT_MODE() DDR|=(1<<TRIGGER)
#define TRIGGERLOW() PORT&=~(1<<TRIGGER)
#define TRIGGERHIGH() PORT|=(1<<TRIGGER)

volatile double time = 0;			// Millisekunden

int main(void);

// Timer/Counter1 Capture Event 
ISR (TIMER1_CAPT_vect)
{
	uint16_t counter = 0;		// Zaehler fuer Ueberlaeufe

	if (TCCR1B & (1 << ICES1))	// wenn in Vorbereitungsphase Flanke auftritt => Beginn Zeitmessung
	{
		TCCR1B &= ~(1 << ICES1);		// fallende Flanke an ICP1 loest IRQ aus
		TCNT1 = 0;						
	}
	else			// Ende der Zeitmessung
	{
		counter = ICR1;					// Zaehlerstand retten
		time = counter/(F_CPU/256.0);	// = Millisekunden
		TCCR1B = 0;						// Timer anhalten
	}
}

int main(void)
{
	char buf[20];
	
	OUTPUT_MODE();
	
	lcd_ini();
	lcd_writetext ("Entfernungsmessung");
	
	TIMSK |= (1 << TICIE1) | (1 << OCIE1A);// Input Capture Interrupt Enable+Output Compare A Match Interrupt Enable
	sei();									// IRQs enable

	while (1)		// endlos
	{
		TRIGGERHIGH();		// Starte Ultraschallsignal
		_delay_us(10);
		TRIGGERLOW();
		
		TCCR1B |= (1 << WGM12);	// CTC Modus
		TCCR1B |= (1 << ICES1);	// steigende Flanke an ICP1
		TCCR1B |= (1 << ICNC1); 	// Noise Canceler
		TCCR1B |= (1 << CS12);		// Prescaler: 1/256
		
		while (TCCR1B);				// warte bis Timer angehalten wurde

		lcd_gotopos (2, 1);
		sprintf(buf, "Zeit: %.4f ms",  time*1000.0);		
		lcd_writetext (buf);
		
		lcd_gotopos (4, 1);
		
		time = time / 2.0 * 34300;		// Einfache Wegstrecke. Die Schallgeschwindigkeit in trockener Luft von 20 C ist 34300 cm/s
		
		if ((time >= 2.0) && (time <= 500))
			sprintf(buf, "Distanz: %.2f cm  ",  time);		
		else
			sprintf(buf, "Fehler Messbereich");
			
		lcd_writetext (buf);
		_delay_ms(200);
	}
	return 0;
}
